/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 *                                                             *
 *  Match.c:                                                   *
 *      This is the heart of this search program, the          *
 *      pattern matcher. It is derived from the fnmatch()      *
 *      library function included with DJGPP. However, it      *
 *      has since been ported to pascal, and then back to      *
 *      C, and significantly improved. It basically bears      *
 *      no resemblance to the original fnmatch() anymore.      *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "search.h"


#define MAX_LIST_ELEMENTS   32


static LPCTSTR STDCALL RangeMatch(LPCTSTR szPattern, TCHAR cTest)
    {
    BOOL  bNegate;
    BOOL  bOK;
    TCHAR c1,c2;

    if ((bNegate = (*szPattern == '!')))
        szPattern++;

    cTest = UpCase(cTest);
    bOK   = FALSE;
    do  {
        c1 = UpCase(*szPattern);
        if ((c1 == '\0') || (c1 == ']') || (c1 == '-'))
            FatalError(TEXT("Invalid regular expression!"));

        c2 = UpCase(*(szPattern+2));
        if ((*(szPattern+1) == '-') && (c2 != '\0') && (c2 != ']'))
            {
            if ((c1 <= cTest) && (cTest <= c2))
                bOK = TRUE;

            szPattern += 2;
            }
        else
            if (c1 == cTest)
                bOK = TRUE;

        } while (*(++szPattern) != ']');

	return (bOK == bNegate) ? NULL : szPattern;
	}

BOOL STDCALL ListMatch(LPCTSTR szPattern, LPCTSTR szName)
	{
	TCHAR   szTempPattern[StrLen(szPattern)];
	INT     curLevel;
    INT     curPattern;
	INT     numPatterns;
    LPCTSTR patterns[MAX_LIST_ELEMENTS];
    INT     lengths[MAX_LIST_ELEMENTS];
    BOOL    finished;

	/* Break up the list into separate patterns */
    patterns[0] = szPattern;
    lengths[0]  = 0;
    numPatterns = 0;
	curLevel    = 1;
    finished    = FALSE;
    while (!finished)
        {
        switch (*szPattern)
            {
            case '\0':
                FatalError("Invalid regular expression!");

            case ',':
                {
                if (curLevel == 1)
                    {
                    szPattern++;

                    numPatterns++;
                    if (numPatterns >= MAX_LIST_ELEMENTS)
                        FatalError("Too many elements in {...}!");

					patterns[numPatterns] = szPattern;
                    lengths[numPatterns]  = 0;
                    continue;
                    }
                else
                    break;
                }

            case '{':
                {
                curLevel++;
                break;
                }

            case '}':
                {
                curLevel--;
                if (curLevel == 0)
                    {
                    numPatterns++;
                    finished = TRUE;
                    }

                break;
                }
            }

		szPattern++;
		lengths[numPatterns]++;
        }

	/* Now, do the actual matching */
    for (curPattern = 0; curPattern < numPatterns; curPattern++)
        {
		StrNCopy(szTempPattern,patterns[curPattern],lengths[curPattern]);
		StrCopy(szTempPattern+lengths[curPattern],szPattern);

		if (FileNameMatch(szTempPattern,szName))
            return TRUE;
		}

    return FALSE;
	}

BOOL STDCALL FileNameMatch(LPCTSTR szPattern, LPCTSTR szName)
	{
	while (TRUE)
		{
        switch (*szPattern)
            {
            case '\0':
                return (*szName == '\0');

            case '?':
                {
                if ((*szName == '\0') || (*szName == '\\'))
                    return FALSE;

                szName++;
                break;
                }

            case '*':
				{
                /* Collapse multiple stars */
                while (*szPattern == '*')
                    szPattern++;

                /* Optimize for pattern with "*" at end */
                if (*szPattern == '\0')
                    return (StrScan(szName,'\\') == NULL);

                /* Optimize for pattern with "*" before "\" */
                else if (*szPattern == '\\')
                    {
                    szName = StrScan(szName,'\\');
                    if (szName == NULL)
                        return FALSE;
                    }

                /* General case, use recursion */
                else
                    {
                    while ((*szName != '\0') && (*szName != '\\'))
                        {
                        if (FileNameMatch(szPattern,szName))
							return TRUE;

                        szName++;
                        }

                    return FALSE;
                    }

                break;
                }

            case '[':
                {
                if ((*szName == '\0') || (*szName == '\\'))
                    return FALSE;

                szPattern = RangeMatch(szPattern+1,*szName);
                if (szPattern == NULL)
                    return FALSE;

                szName++;
                break;
                }

            case '{':
                return ListMatch(szPattern+1,szName);

            default:
                {
                if (UpCase(*szPattern) != UpCase(*szName))
                    return FALSE;

                szName++;
                break;
                }
            }

        szPattern++;
        }
    }
