/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 *                                                             *
 *  Search.c:                                                  *
 *      Main program source. This is a console application,    *
 *      compiled with the mingw32 port of gcc. It has not      *
 *      been tested (compiled) with any other compiler.        *
 *                                                             *
 *      If you find any bugs, or make any changes to this      *
 *      source, please email me the details.                   *
 *                                                             *
 *         www:   www.geocities.com/SiliconValley/7259/        *
 *         email: bernie-t@geocities.com                       *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "search.h"


#ifdef __GCC__
    int _CRT_glob = 0;          /* Turn off commandline globbing */
#endif

SEARCH_IN  SearchIn;
SEARCH_OUT SearchOut;


VOID STDCALL FatalError(LPCTSTR szMessage)
    {
    WriteF(stderr,TEXT("\n%s\n"),szMessage);
    exit(EXIT_FATAL_ERROR);
    }

VOID STDCALL FormatPath(LPTSTR szPath)
    {
    while (*szPath)
        {
        if (*szPath == '/') *szPath = '\\';
        szPath++;
        }
    }

LPTSTR STDCALL StrDuplicate(LPCTSTR szString)
    {
    if (szString)
        {
        LPTSTR szNewString = LocalAlloc(LMEM_FIXED,
                                        (StrLen(szString)+1) * sizeof(TCHAR));
        if (!szNewString) FatalError(TEXT("Not enough memory!"));

        return StrCopy(szNewString,szString);
        }
    else
        return NULL;
    }

VOID STDCALL WritePathF(FILE    *file,
                        LPCTSTR szPath,
                        INT     charsWritten)
    {
    INT len    = StrLen(szPath);
    INT maxLen = CONSOLE_WIDTH-1 - charsWritten;

    if (len > maxLen)
        {
        if (maxLen >= 6)
            WriteF(file,TEXT("%c%c%c...%-*s"),
                        szPath[0],szPath[1],szPath[2],
                        maxLen-6,&(szPath[len-maxLen+6]));
        else
            WriteF(file,TEXT("%-*s"),maxLen,TEXT("..."));
        }
    else
        WriteF(file,TEXT("%-*s"),maxLen,szPath);
    }

VOID STDCALL CheckEscape()
    {
    if (_kbhit())
        SearchOut.bAborted = (_getch() == ESCAPE_KEY);
    }

VOID STDCALL DisplayStatus(LPCTSTR szPath)
    {
    WriteF(stderr,TEXT("\r%6d/%6d "),SearchOut.dwNumFound,SearchOut.dwNumSearched);
    WritePathF(stderr,szPath,6+1+6+1);
    fflush(stderr);
    }

VOID STDCALL UpdateStatus()
    {
    TCHAR szTitle[64];

    WriteF(stderr,TEXT("\r%6d/%6d"),SearchOut.dwNumFound,SearchOut.dwNumSearched);

    WriteS(szTitle,TEXT("Searching %d/%d..."),SearchOut.dwNumFound,SearchOut.dwNumSearched);
    SetConsoleTitle(szTitle);
    }

VOID STDCALL AddToFoundList(LPCTSTR         szFileName,
                            LPCTSTR         szArchiveName,
                            DWORD           dwFileAttributes,
                            CONST FILETIME *ftCreationTime,
                            CONST FILETIME *ftLastAccessTime,
                            CONST FILETIME *ftLastWriteTime,
                            DWORD           dwFileSizeHigh,
                            DWORD           dwFileSizeLow)
    {
    PFOUND_ITEM pNewItem;

    /* Create new list item */
    pNewItem = LocalAlloc(LMEM_FIXED,sizeof(FOUND_ITEM));
    if (!pNewItem) FatalError(TEXT("Not enough memory!"));

    /* Set list item fields */
    pNewItem->pNext            = NULL;
    pNewItem->szFileName       = StrDuplicate(szFileName);
    pNewItem->szArchiveName    = StrDuplicate(szArchiveName);
    pNewItem->dwFileAttributes = dwFileAttributes;
    pNewItem->ftCreationTime   = *ftCreationTime;
    pNewItem->ftLastAccessTime = *ftLastAccessTime;
    pNewItem->ftLastWriteTime  = *ftLastWriteTime;
    pNewItem->dwFileSizeHigh   = dwFileSizeHigh;
    pNewItem->dwFileSizeLow    = dwFileSizeLow;

    /* Update SearchOut */
    if (!SearchOut.pListBeg)
        SearchOut.pListBeg = pNewItem;
    else
        SearchOut.pListEnd->pNext = pNewItem;
    SearchOut.pListEnd = pNewItem;
    SearchOut.dwNumFound++;
    }

VOID STDCALL CleanFoundList()
    {
    PFOUND_ITEM pItem;

    while (SearchOut.pListBeg)
        {
        pItem = SearchOut.pListBeg;
        SearchOut.pListBeg = SearchOut.pListBeg->pNext;

        LocalFree(pItem->szFileName);
        LocalFree(pItem->szArchiveName);
        LocalFree(pItem);
        }

    SearchOut.dwNumFound = 0;
    SearchOut.pListBeg = NULL;
    SearchOut.pListEnd = NULL;
    }

VOID STDCALL WriteFoundList()
    {
    PFOUND_ITEM pItem;

    pItem = SearchOut.pListBeg;
    while (pItem)
        {
        if (pItem->szArchiveName)
            WriteF(stdout,TEXT("%s - "),pItem->szArchiveName);
        WriteF(stdout,TEXT("%s\n"),pItem->szFileName);

        pItem = pItem->pNext;
        }
    }


static VOID STDCALL SearchDOS(LPCTSTR szPath)
    {
    WIN32_FIND_DATA findData;
    HANDLE          handle;
    TCHAR           szTempPath[PATH_MAX+1];

    DisplayStatus(szPath);

    StrCopy(szTempPath,szPath);
    StrCat(szTempPath,TEXT("*"));

    handle = FindFirstFile(szTempPath,&findData);
    if (handle != INVALID_HANDLE_VALUE)
        {
		do  {
			if (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
                {
                if ((StrComp(findData.cFileName,TEXT("."))  != 0) &&
                    (StrComp(findData.cFileName,TEXT("..")) != 0))
                        {
                        StrCopy(szTempPath,szPath);
                        StrCat(szTempPath,findData.cFileName);
                        StrCat(szTempPath,TEXT("\\"));
                        SearchDOS(szTempPath);
                        }
                }
            else
                {
                SearchOut.dwNumSearched++;

                StrCopy(szTempPath,szPath);
                StrCat(szTempPath,findData.cFileName);

				if (((findData.dwFileAttributes | SearchIn.dwFileAttributes) == findData.dwFileAttributes) &&
                   FileNameMatch(SearchIn.szSpecs,findData.cFileName))
                    {
                    AddToFoundList(szTempPath,NULL,
                                   findData.dwFileAttributes,
                                   &(findData.ftCreationTime),
                                   &(findData.ftLastAccessTime),
                                   &(findData.ftLastWriteTime),
                                   findData.nFileSizeHigh,
                                   findData.nFileSizeLow);
                    }

                if (SearchIn.bInArch)
                    {
                    if (FileNameMatch(TEXT("*.ZIP"),findData.cFileName))
                        SearchZIP(szTempPath);
                    else if (FileNameMatch(TEXT("*.ARJ"),findData.cFileName))
                        SearchARJ(szTempPath);
					else if (FileNameMatch(TEXT("*.RAR"),findData.cFileName))
                        SearchRAR(szTempPath);
                    else if (FileNameMatch(TEXT("*.CAB"),findData.cFileName))
                        SearchCAB(szTempPath);
                    else if (FileNameMatch(TEXT("*.LZH"),findData.cFileName))
                        SearchLZH(szTempPath);
                    else if (FileNameMatch(TEXT("*.ARC"),findData.cFileName))
                        SearchARC(szTempPath);
                    else if (FileNameMatch(TEXT("*.PAK"),findData.cFileName))
                        {
                        SearchPAK(szTempPath);  /* Quake PAK */
                        SearchARC(szTempPath);  /* PKPAK     */
                        }
                    }
				}

			CheckEscape();
			} while (!SearchOut.bAborted && FindNextFile(handle,&findData));

        FindClose(handle);
		}

	UpdateStatus();
	}


static VOID STDCALL Initialise()
    {
    SearchIn.szSpecs[0]       = '\0';
    SearchIn.szRoot[0]        = '\0';
    SearchIn.dwFileAttributes = 0;
    SearchIn.bPause           = FALSE;
    SearchIn.bAllDrives       = FALSE;
    SearchIn.bInArch          = FALSE;

    SearchOut.dwNumFound      = 0;
    SearchOut.dwNumSearched   = 0;
    SearchOut.pListBeg        = NULL;
    SearchOut.pListEnd        = NULL;
    SearchOut.bAborted        = FALSE;
    }

static VOID STDCALL DisplayHelp()
    {
    WriteF(stderr,
           TEXT("SEARCH.EXE pattern [/C] [/D] [/A:attr] [>outfile]\n")
           TEXT("\n")
           TEXT("  pattern   A regular expression describing the files to find:\n")
           TEXT("              *           Matches zero or more characters\n")
           TEXT("              ?           Matches one single character\n")
           TEXT("              [abc0-9]    Matches a, b, c, and 0 thru 9\n")
           TEXT("              [!abc0-9]   Matches any characters except a, b, c, and 0 thru 9\n")
           TEXT("              {str,...}   Matches any string in the list\n")
           TEXT("\n")
           TEXT("  /C        Search inside compressed archives. Currently supports:\n")
           TEXT("              .ZIP  .ARJ  .RAR  .CAB  .LZH  .ARC  .PAK\n")
           TEXT("\n")
           TEXT("  /D        Search on all available drives (C: thru Z:).\n")
           TEXT("\n")
           TEXT("  /A:attr   Search for files with the following attributes set:\n")
           TEXT("              S  System       H  Hidden\n")
           TEXT("              R  Read-only    A  Archive\n")
           TEXT("\n")
           TEXT("  >outfile  Save list of matching files into outfile.\n")
           TEXT("            Using >>outfile appends the list to outfile.\n"));

    exit(EXIT_SYNTAX);
    }

static VOID STDCALL ProcessArguments()
    {
    TCHAR  szFileSpecs[PATH_MAX+1];
    LPTSTR szArgs = GetCommandLine();
    LPTSTR szSpecs;

    /* Skip name of executable */
    while ((*szArgs != '\0') &&
           (*szArgs != '\t') &&
           (*szArgs != ' '))
        szArgs++;

    /* Now process the arguments */
    szFileSpecs[0] = '\0';
    szSpecs = szFileSpecs;
    while (*szArgs != '\0')
        switch (*szArgs)
            {
            case '\t': case ' ':        /* Whitespace */
                {
                szArgs++;
                break;
                }

            case '/': case '-':         /* Switch */
                {
                szArgs++;
                switch (UpCase(*szArgs))
                    {
                    case '?': DisplayHelp();              break;
                    case 'P': SearchIn.bPause     = TRUE; break;
                    case 'D': SearchIn.bAllDrives = TRUE; break;
                    case 'C': SearchIn.bInArch    = TRUE; break;
                    case 'A':
                        {
                        if (*(szArgs+1) == ':') szArgs++;

                        do  {
                            szArgs++;

                            switch (UpCase(*szArgs))
                                {
                                case 'R': SearchIn.dwFileAttributes |= FILE_ATTRIBUTE_READONLY; break;
                                case 'H': SearchIn.dwFileAttributes |= FILE_ATTRIBUTE_HIDDEN;   break;
                                case 'S': SearchIn.dwFileAttributes |= FILE_ATTRIBUTE_SYSTEM;   break;
                                case 'A': SearchIn.dwFileAttributes |= FILE_ATTRIBUTE_ARCHIVE;  break;
                                default:  FatalError(TEXT("Invalid attribute after /A !"));
                                }
                            } while ((*(szArgs+1) != '\0') &&
                                     (*(szArgs+1) != '\t') &&
                                     (*(szArgs+1) != ' '));
                        break;
                        }

                    default:
                        FatalError(TEXT("Invalid option!"));
                    }
                szArgs++;
                break;
                }

            default:                    /* Filespec characters */
                {
                BOOL bQuoted = FALSE;

                if (szFileSpecs[0] != '\0')
                    FatalError(TEXT("Multiple patterns not allowed!"));

                while ((*szArgs) &&
                       (bQuoted || ((*szArgs != '\t') &&
                                    (*szArgs != ' ') &&
                                    (*szArgs != '/'))))
                    {
                    if ((*szArgs == '\"') || (*szArgs == '\''))
                        {
                        if (*(szArgs+1) == *szArgs)
                            *szSpecs++ = *szArgs++;
                        else
                            bQuoted = ! bQuoted;
                        szArgs++;
                        }
                    else
                        *szSpecs++ = *szArgs++;
                    }

                *szSpecs = '\0';
                break;
                }
            }

    /* Look for '\' or ':' */
    szSpecs = StrRScan(szFileSpecs,'\\');
    if (!szSpecs) szSpecs = StrScan(szFileSpecs,':');

    /* Now split into Root and Specs */
    if (!szSpecs)
        {
        StrCopy(SearchIn.szSpecs,szFileSpecs);
        GetFullPathName(TEXT(".\\"),PATH_MAX,SearchIn.szRoot,&szSpecs);
        }
    else
        {
        /* Split into Specs */
        szSpecs++;
        StrCopy(SearchIn.szSpecs,szSpecs);

        /* Split into Root */
        *szSpecs = '\0';
        GetFullPathName(szFileSpecs,PATH_MAX,SearchIn.szRoot,&szSpecs);
        }

    /* Did we get filespecs? */
    if (SearchIn.szSpecs[0] == '\0')
        FatalError(TEXT("No pattern specified!"));

    /* Do we have a valid starting directory? */
    if (SearchIn.szRoot[0] == '\0')
        FatalError(TEXT("Invalid starting directory!"));
    }

static VOID STDCALL DoSearch()
    {
    SetConsoleTitle(TEXT("Searching..."));

    if (SearchIn.bAllDrives)
        {
        }
    else
        {
        WriteF(stderr,TEXT("Searching for \"%s\"")
                      TEXT(" starting from \"%s\"")
                      TEXT(" (Press ESC to abort)\n"),
                      SearchIn.szSpecs,SearchIn.szRoot);

        SearchDOS(SearchIn.szRoot);
        }

    WriteF(stderr,TEXT("\r%*s\n"),CONSOLE_WIDTH-1,TEXT(""));
    }

static VOID STDCALL DisplayResults()
    {
    SetConsoleTitle(TEXT("Listing..."));

    if (SearchOut.dwNumFound > 0)
        {
        WriteFoundList();
        WriteF(stderr,TEXT("Found %d matching file(s) out of %d file(s)\n"),
                      SearchOut.dwNumFound,
                      SearchOut.dwNumSearched);

        CleanFoundList();
        }
    else
        WriteF(stderr,TEXT("No matching files were found out of %d file(s)\n"),
                      SearchOut.dwNumSearched);
    }


int main()
    {
    WriteF(stderr,
           TEXT("Search v2.0 (win32) - Utility to search for a specified file\n")
           TEXT("Copyright (C) 1996-1998, Bernhard Tschirren\n")
           TEXT("\n"));

    Initialise();
    ProcessArguments();
    DoSearch();
    DisplayResults();

    return EXIT_ALL_OK;
    }
