/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <conio.h>
#include <io.h>

#ifdef UNICODE
    #error Sorry, the UNICODE Win32 API functions dont seem to work!
#endif


#define EXIT_ALL_OK         (0x00)
#define EXIT_SYNTAX         (0x01)
#define EXIT_FATAL_ERROR    (0xFF)

#define CONSOLE_WIDTH       (80)
#define CONSOLE_HEIGHT      (25)

#define ESCAPE_KEY          (27)

#ifdef UNICODE
    #define WriteF(args...)     fwprintf(args)
    #define WriteS(args...)     swprintf(args)
    #define UpCase(x)           towupper(x)
    #define StrLen(x)           wcslen(x)
    #define StrCopy(x,y)        wcscpy(x,y)
    #define StrNCopy(x,y,z)     wcsncpy(x,y,z)
    #define StrCat(x,y)         wcscat(x,y)
    #define StrComp(x,y)        wcscmp(x,y)
    #define StrScan(x,y)        wcschr(x,y)
    #define StrRScan(x,y)       wcsrchr(x,y)
#else
    #define WriteF(args...)     fprintf(args)
    #define WriteS(args...)     sprintf(args)
    #define UpCase(x)           toupper(x)
    #define StrLen(x)           strlen(x)
    #define StrCopy(x,y)        strcpy(x,y)
    #define StrNCopy(x,y,z)     strncpy(x,y,z)
    #define StrCat(x,y)         strcat(x,y)
    #define StrComp(x,y)        strcmp(x,y)
    #define StrScan(x,y)        strchr(x,y)
	#define StrRScan(x,y)       strrchr(x,y)
#endif

#ifdef __MINGW32__
	#define ATTRIB_NORETURN     __attribute__ ((noreturn))
	#define ATTRIB_PACKED		__attribute__ ((packed))
#else
	#define ATTRIB_NORETURN
	#define ATTRIB_PACKED
#endif


typedef struct _FOUND_ITEM
    {
    struct _FOUND_ITEM *pNext;
    LPTSTR              szFileName;
    LPTSTR              szArchiveName;
    DWORD               dwFileAttributes;
    FILETIME            ftCreationTime;
    FILETIME            ftLastAccessTime;
    FILETIME            ftLastWriteTime;
    DWORD               dwFileSizeHigh;
    DWORD               dwFileSizeLow;
    } FOUND_ITEM, *PFOUND_ITEM;

typedef struct _SEARCH_IN
    {
    TCHAR szSpecs[PATH_MAX+1];
    TCHAR szRoot[PATH_MAX+1];
    DWORD dwFileAttributes;
    BOOL  bPause;
    BOOL  bAllDrives;
    BOOL  bInArch;
    } SEARCH_IN, *PSEARCH_IN;

typedef struct _SEARCH_OUT
    {
    DWORD       dwNumFound;
    DWORD       dwNumSearched;
    PFOUND_ITEM pListBeg;
    PFOUND_ITEM pListEnd;
    BOOL        bAborted;
    } SEARCH_OUT, *PSEARCH_OUT;

/* Globals */
extern SEARCH_IN  SearchIn;
extern SEARCH_OUT SearchOut;

/* Search.c */
VOID STDCALL FatalError(LPCTSTR szMessage) ATTRIB_NORETURN;
VOID STDCALL FormatPath(LPTSTR szPath);
LPTSTR STDCALL StrDuplicate(LPCTSTR szString);

/* Search.c */
VOID STDCALL CheckEscape();
VOID STDCALL DisplayStatus(LPCTSTR szPath);
VOID STDCALL UpdateStatus();

/* Search.c */
VOID STDCALL AddToFoundList(LPCTSTR         szFileName,
                            LPCTSTR         szArchiveName,
                            DWORD           dwFileAttributes,
                            CONST FILETIME *ftCreationTime,
                            CONST FILETIME *ftLastAccessTime,
                            CONST FILETIME *ftLastWriteTime,
                            DWORD           dwFileSizeHigh,
                            DWORD           dwFileSizeLow);

/* Match.c */
BOOL STDCALL FileNameMatch(LPCTSTR szPattern, LPCTSTR szName);

/* Archive Searchers */
VOID STDCALL SearchZIP(LPCTSTR szArchiveName);
VOID STDCALL SearchARJ(LPCTSTR szArchiveName);
VOID STDCALL SearchRAR(LPCTSTR szArchiveName);
VOID STDCALL SearchCAB(LPCTSTR szArchiveName);
VOID STDCALL SearchLZH(LPCTSTR szArchiveName);
VOID STDCALL SearchARC(LPCTSTR szArchiveName);
VOID STDCALL SearchPAK(LPCTSTR szArchiveName);
