/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 *                                                             *
 *  SearchARC.c:                                               *
 *      Search inside ARC archives. File format details        *
 *      obtained from Wotsit's File Format Collection,         *
 *      (http://www.wotsit.org/).                              *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "search.h"


#define SIGNATURE           0x1A

typedef struct
	{
	BYTE  signature         ATTRIB_PACKED;
	BYTE  dataType			ATTRIB_PACKED;
	CHAR  szBaseName[13]	ATTRIB_PACKED;
	DWORD packSize			ATTRIB_PACKED;
	WORD  date				ATTRIB_PACKED;
	WORD  time				ATTRIB_PACKED;
	WORD  crc16				ATTRIB_PACKED;
	DWORD origSize			ATTRIB_PACKED;
	} HEADER;


VOID STDCALL SearchARC(LPCTSTR szArchiveName)
	{
	FILE   *file;
	DWORD   fileSize;
	HEADER  header;
	DWORD   headerPos;

	DisplayStatus(szArchiveName);

	file = fopen(szArchiveName,"rb");
	if (!file) return;

	headerPos = 0;
	fileSize  = _filelength(_fileno(file));

	do  {
		/* Locate file header */
		if (fseek(file,headerPos,SEEK_SET) != 0)
			break;

		/* Read file header */
		if (fread(&header,sizeof(HEADER),1,file) != 1)
			break;

		/* Check validity of header */
		if ((header.signature != SIGNATURE) ||
			(header.dataType  == 0))
			break;

		/* Administrivia */
		SearchOut.dwNumSearched++;

		/* Add to found list (if appropriate) */
		if (FileNameMatch(SearchIn.szSpecs,header.szBaseName))
			{
			FILETIME fileTime;
			DosDateTimeToFileTime(header.date,
								  header.time,
								  &fileTime);

			AddToFoundList(header.szBaseName,
						   szArchiveName,
						   0,
						   &fileTime,
						   &fileTime,
						   &fileTime,
						   0,
						   header.origSize);
			}

		headerPos += sizeof(HEADER);
		headerPos += header.packSize;

		CheckEscape();
		} while (!SearchOut.bAborted && (headerPos < fileSize));

	fclose(file);

	UpdateStatus();
	}
