/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 *                                                             *
 *  SearchARJ.c:                                               *
 *      Search inside ARJ archives. File format details        *
 *      were obtained from "technote.doc" included with the    *
 *      ARJ Archiver (version 2.60).                           *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "search.h"


#define SIGNATURE       0xEA60
#define BUFFER_SIZE     4096


typedef struct
    {
    WORD  signature     ATTRIB_PACKED;
    WORD  headerSize    ATTRIB_PACKED;
    BYTE  firstSize     ATTRIB_PACKED;
    BYTE  version       ATTRIB_PACKED;
    BYTE  minVersion    ATTRIB_PACKED;
    BYTE  hostOS        ATTRIB_PACKED;
    BYTE  flags         ATTRIB_PACKED;
    BYTE  method        ATTRIB_PACKED;
    BYTE  fileType      ATTRIB_PACKED;
    BYTE  reserved      ATTRIB_PACKED;
    WORD  date          ATTRIB_PACKED;
    WORD  time          ATTRIB_PACKED;
    DWORD packSize      ATTRIB_PACKED;
    DWORD origSize      ATTRIB_PACKED;
    } HEADER;

typedef union
    {
    CHAR   buffer[BUFFER_SIZE];
    HEADER header;
    } DATA;


VOID STDCALL SearchARJ(LPCTSTR szArchiveName)
	{
	FILE   *file;
    DWORD   fileSize;
    DATA    data;
    LPSTR   szFullName;
    LPSTR   szBaseName;
	DWORD   headerPos;

	DisplayStatus(szArchiveName);

	file = fopen(szArchiveName,"rb");
	if (!file) return;

	/* Read archive header */
    if (fread(&data,4,1,file) != 1)
		{
		fclose(file);
		return;
		}

	/* Check validity of file */
    if (data.header.signature != SIGNATURE)
		{
		fclose(file);
		return;
		}

    headerPos = 4 + data.header.headerSize + 4 + 2;
    fileSize  = _filelength(_fileno(file));

	do  {
		/* Locate file header */
		if (fseek(file,headerPos,SEEK_SET) != 0)
			break;

		/* Read ARJ header and filename into buffer */
        if (fread(&data,sizeof(DATA),1,file) != 1)
			break;

		/* Check validity of file */
        if (data.header.signature != SIGNATURE)
			break;

		/* Administrivia */
        szFullName = &(data.buffer[data.header.firstSize + 4]);
		FormatPath(szFullName);
		SearchOut.dwNumSearched++;

		/* Strip path */
		szBaseName = StrRScan(szFullName,'\\');
		if (!szBaseName) szBaseName = szFullName; else szBaseName++;

		/* Add to found list (if appropriate) */
		if (FileNameMatch(SearchIn.szSpecs,szBaseName))
			{
			FILETIME fileTime;
            DosDateTimeToFileTime(data.header.date,
                                  data.header.time,
                                  &fileTime);

			AddToFoundList(szFullName,
						   szArchiveName,
						   0,
						   &fileTime,
						   &fileTime,
						   &fileTime,
						   0,
                           data.header.origSize);
			}

		headerPos += 4;
		headerPos += data.header.headerSize+4+2;
		headerPos += data.header.packSize;

		CheckEscape();
		} while (!SearchOut.bAborted && (headerPos < fileSize));

	fclose(file);

	UpdateStatus();
	}
