/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 *                                                             *
 *  SearchCAB.c:                                               *
 *      Search inside Microsoft CAB archives. File format      *
 *      details obtained from Wotsit's File Format Collection, *
 *      (http://www.wotsit.org/).                              *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "search.h"


#define SIGNATURE		0x4643534DL
#define VERSION			0x0103

typedef struct
	{
    DWORD signature     ATTRIB_PACKED;
    DWORD headerCheck   ATTRIB_PACKED;
    DWORD archSize      ATTRIB_PACKED;
	DWORD foldersCheck  ATTRIB_PACKED;
	DWORD fileList      ATTRIB_PACKED;
    DWORD filesCheck    ATTRIB_PACKED;
    WORD  version       ATTRIB_PACKED;
    WORD  numFolders    ATTRIB_PACKED;
    WORD  numFiles      ATTRIB_PACKED;
    WORD  flags         ATTRIB_PACKED;
    WORD  setID         ATTRIB_PACKED;
    WORD  cabinetNum    ATTRIB_PACKED;
	} ARCH_HEADER;

typedef struct
	{
    DWORD origSize      ATTRIB_PACKED;
    DWORD origOffset    ATTRIB_PACKED;
    WORD  flags         ATTRIB_PACKED;
    WORD  date          ATTRIB_PACKED;
    WORD  time          ATTRIB_PACKED;
    WORD  attribs       ATTRIB_PACKED;
	} FILE_HEADER;


VOID STDCALL SearchCAB(LPCTSTR szArchiveName)
	{
	FILE        *file;
	ARCH_HEADER  archHeader;
	FILE_HEADER  fileHeader;
	DWORD        headerPos;
	CHAR         szFullName[MAX_PATH+1];
	LPSTR		 szBaseName;

	DisplayStatus(szArchiveName);

	file = fopen(szArchiveName,"rb");
	if (!file) return;

	/* Read archive header */
    if (fread(&archHeader,sizeof(ARCH_HEADER),1,file) != 1)
		{
		fclose(file);
		return;
		}

	/* Check archive validity */
	if ((archHeader.signature  != SIGNATURE) ||
		(archHeader.version    != VERSION)   ||
		(archHeader.numFolders == 0)         ||
		(archHeader.numFiles   == 0))
		{
		fclose(file);
		return;
		}

	headerPos = archHeader.fileList;

	do  {
		/* Locate file header */
		if (fseek(file,headerPos,SEEK_SET) != 0)
			break;

		/* Read file header */
        if (fread(&fileHeader,sizeof(FILE_HEADER),1,file) != 1)
			break;

        /* Read file name - As many characters as possible */
        if (fread(szFullName,1,MAX_PATH+1,file) == -1)
			break;

		/* Administrivia */
		FormatPath(szFullName);
		SearchOut.dwNumSearched++;

		/* Strip path */
		szBaseName = StrRScan(szFullName,'\\');
		if (!szBaseName) szBaseName = szFullName; else szBaseName++;

		/* Add to found list (if appropriate) */
		if (FileNameMatch(SearchIn.szSpecs,szBaseName))
			{
			FILETIME fileTime;
			DosDateTimeToFileTime(fileHeader.date,
								  fileHeader.time,
								  &fileTime);

			AddToFoundList(szFullName,
						   szArchiveName,
						   0,
						   &fileTime,
						   &fileTime,
						   &fileTime,
						   0,
						   fileHeader.origSize);
			}

		headerPos += sizeof(FILE_HEADER);
		headerPos += StrLen(szFullName) + 1;

		CheckEscape();
		} while (!SearchOut.bAborted && (--archHeader.numFiles > 0));

	fclose(file);

	UpdateStatus();
	}
