/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 *                                                             *
 *  SearchLZH.c:                                               *
 *      Search inside LZH archives. File format details        *
 *      obtained from Wotsit's File Format Collection,         *
 *      (http://www.wotsit.org/).                              *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "search.h"


#define METHOD_0		'-'
#define METHOD_1    	'l'
#define METHOD_2    	'h'
#define METHOD_4    	'-'

typedef struct
	{
	BYTE  headerSize	ATTRIB_PACKED;
	BYTE  headerCheck	ATTRIB_PACKED;
	CHAR  method[5]		ATTRIB_PACKED;
	DWORD packSize		ATTRIB_PACKED;
	DWORD origSize		ATTRIB_PACKED;
	WORD  date			ATTRIB_PACKED;
	WORD  time			ATTRIB_PACKED;
	WORD  attribs		ATTRIB_PACKED;
	BYTE  nameLength	ATTRIB_PACKED;
	} HEADER;


VOID STDCALL SearchLZH(LPCTSTR szArchiveName)
	{
	FILE   *file;
	DWORD   fileSize;
	HEADER  header;
	DWORD   headerPos;
	CHAR    szFullName[MAX_PATH+1];
	LPSTR	szBaseName;

	DisplayStatus(szArchiveName);

	file = fopen(szArchiveName,"rb");
	if (!file) return;

	headerPos = 0;
	fileSize  = _filelength(_fileno(file));

	do  {
		/* Locate file header */
		if (fseek(file,headerPos,SEEK_SET) != 0)
			break;

		/* Read file header */
        if (fread(&header,sizeof(HEADER),1,file) != 1)
			break;

		/* Check validity of header */
		if ((header.method[0] != METHOD_0) ||
			(header.method[1] != METHOD_1) ||
			(header.method[2] != METHOD_2) ||
			(header.method[4] != METHOD_4))
			break;

		/* Read file name */
        if (fread(szFullName,header.nameLength,1,file) != 1)
			break;

		/* Administrivia */
		szFullName[header.nameLength] = '\0';
		FormatPath(szFullName);
		SearchOut.dwNumSearched++;

		/* Strip path */
		szBaseName = StrRScan(szFullName,'\\');
		if (!szBaseName) szBaseName = szFullName; else szBaseName++;

		/* Add to found list (if appropriate) */
		if (FileNameMatch(SearchIn.szSpecs,szBaseName))
			{
			FILETIME fileTime;
			DosDateTimeToFileTime(header.date,
								  header.time,
								  &fileTime);

			AddToFoundList(szFullName,
						   szArchiveName,
						   0,
						   &fileTime,
						   &fileTime,
						   &fileTime,
						   0,
						   header.origSize);
			}

		headerPos += header.headerSize+2;
		headerPos += header.packSize;

		CheckEscape();
		} while (!SearchOut.bAborted && (headerPos < fileSize));

	fclose(file);

	UpdateStatus();
	}
