/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 *                                                             *
 *  SearchPAK.c:                                               *
 *      Search inside Quake 1 and 2 PAK archives. File format  *
 *      details were obtained from the Quake2 utility          *
 *      sources released by ID software. Specifically, from    *
 *      the unpack utility written in Java.                    *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "search.h"


#define SIGNATURE_0			'P'
#define SIGNATURE_1			'A'
#define SIGNATURE_2			'C'
#define SIGNATURE_3			'K'

typedef struct
	{
	CHAR  signature[4]		ATTRIB_PACKED;
	DWORD dirOffset			ATTRIB_PACKED;
	DWORD dirLength			ATTRIB_PACKED;
	} ARCH_HEADER;

typedef struct
	{
	CHAR  szFullName[56]	ATTRIB_PACKED;
	DWORD filePos			ATTRIB_PACKED;
	DWORD fileSize			ATTRIB_PACKED;
	} FILE_HEADER;


VOID STDCALL SearchPAK(LPCTSTR szArchiveName)
	{
	FILE        *file;
	ARCH_HEADER  archHeader;
	FILE_HEADER  fileHeader;
	LPSTR        szBaseName;
	DWORD        numEntries;

	DisplayStatus(szArchiveName);

	file = fopen(szArchiveName,"rb");
	if (!file) return;

	/* Read archive header */
	if (fread(&archHeader,sizeof(ARCH_HEADER),1,file) != 1)
		{
		fclose(file);
		return;
		}

	/* Check validity of archive */
	if ((archHeader.signature[0] != SIGNATURE_0) ||
		(archHeader.signature[1] != SIGNATURE_1) ||
		(archHeader.signature[2] != SIGNATURE_2) ||
        (archHeader.signature[3] != SIGNATURE_3))
		{
		fclose(file);
		return;
		}

    /* Locate file directory */
    if (fseek(file,archHeader.dirOffset,SEEK_SET) != 0)
        {
        fclose(file);
        return;
        }

	numEntries = archHeader.dirLength / sizeof(FILE_HEADER);
	for (; (!SearchOut.bAborted) && (numEntries > 0); numEntries--)
		{
		/* Read file header */
		if (fread(&fileHeader,sizeof(FILE_HEADER),1,file) != 1)
			break;

		/* Administrivia */
		FormatPath(fileHeader.szFullName);
		SearchOut.dwNumSearched++;

		/* Strip path */
		szBaseName = StrRScan(fileHeader.szFullName,'\\');
		if (!szBaseName) szBaseName = fileHeader.szFullName; else szBaseName++;

		/* Add to found list (if appropriate) */
		if (FileNameMatch(SearchIn.szSpecs,szBaseName))
			{
			FILETIME fileTime;
			DosDateTimeToFileTime(0,0,&fileTime);

			AddToFoundList(fileHeader.szFullName,
						   szArchiveName,
						   0,
						   &fileTime,
						   &fileTime,
						   &fileTime,
						   0,
						   fileHeader.fileSize);
			}
		}

	fclose(file);

	UpdateStatus(szArchiveName);
	}
