/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 *                                                             *
 *  SearchRAR.c:                                               *
 *      Search inside RAR archives (version 1.5 and above).    *
 *      File format details obtained from "technote.txt"       *
 *      included with the RAR archiver (version 2.04).         *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "search.h"


#define SIGNATURE_SIZE  7
#define SIGNATURE_0   	0x52
#define SIGNATURE_1     0x61
#define SIGNATURE_2     0x72
#define SIGNATURE_3     0x21
#define SIGNATURE_4     0x1A
#define SIGNATURE_5     0x07
#define SIGNATURE_6     0x00

typedef struct
	{
	DWORD packSize		ATTRIB_PACKED;
	DWORD origSize		ATTRIB_PACKED;
	BYTE  hostOS		ATTRIB_PACKED;
	DWORD crc32			ATTRIB_PACKED;
    WORD  date          ATTRIB_PACKED;
    WORD  time          ATTRIB_PACKED;
	BYTE  version		ATTRIB_PACKED;
	BYTE  method		ATTRIB_PACKED;
	WORD  nameLength	ATTRIB_PACKED;
	DWORD flags			ATTRIB_PACKED;
	} HEADER;

typedef struct
	{
	WORD headerCRC		ATTRIB_PACKED;
	BYTE headerType		ATTRIB_PACKED;
	WORD headerFlags	ATTRIB_PACKED;
	WORD headerSize		ATTRIB_PACKED;
	} BLOCK;


VOID STDCALL SearchRAR(LPCTSTR szArchiveName)
	{
	FILE  *file;
	DWORD  fileSize;
	DWORD  headerPos;
	HEADER header;
	BLOCK  block;
    CHAR   signature[SIGNATURE_SIZE];
    CHAR   szFullName[MAX_PATH+1];
	LPSTR  szBaseName;

	DisplayStatus(szArchiveName);

	file = fopen(szArchiveName,"rb");
	if (!file) return;

    if (fread(signature,SIGNATURE_SIZE,1,file) != 1)
		{
		fclose(file);
		return;
		}

	if ((signature[0] != SIGNATURE_0) ||
		(signature[1] != SIGNATURE_1) ||
		(signature[2] != SIGNATURE_2) ||
		(signature[3] != SIGNATURE_3) ||
		(signature[4] != SIGNATURE_4) ||
        (signature[5] != SIGNATURE_5) ||
        (signature[6] != SIGNATURE_6))
		{
		fclose(file);
		return;
		}

    headerPos = SIGNATURE_SIZE;
    fileSize  = _filelength(_fileno(file));

	do  {
		/* Locate file header */
		if (fseek(file,headerPos,SEEK_SET) != 0)
			break;

		/* Read block header */
        if (fread(&block,sizeof(BLOCK),1,file) != 1)
			break;

        if (block.headerType == 0x74)
			{
			/* Read file header */
            if (fread(&header,sizeof(HEADER),1,file) != 1)
				break;

			/* Read file name */
            if (fread(szFullName,header.nameLength,1,file) != 1)
				break;

			/* Administrivia */
			szFullName[header.nameLength] = '\0';
			FormatPath(szFullName);
			SearchOut.dwNumSearched++;

			/* Strip path */
			szBaseName = StrRScan(szFullName,'\\');
			if (!szBaseName) szBaseName = szFullName; else szBaseName++;

			/* Add to found list (if appropriate) */
			if (FileNameMatch(SearchIn.szSpecs,szBaseName))
				{
				FILETIME fileTime;
				DosDateTimeToFileTime(header.date,
									  header.time,
									  &fileTime);

				AddToFoundList(szFullName,
							   szArchiveName,
							   0,
							   &fileTime,
							   &fileTime,
							   &fileTime,
							   0,
							   header.origSize);
				}

			headerPos += block.headerSize;
			headerPos += header.packSize;
			}
		else
            {
            headerPos += block.headerSize;

            if (block.headerFlags & 0x8000)
                {
                WORD addedSize;

                if (fread(&addedSize,sizeof(WORD),1,file) != 1)
                    break;

                headerPos += addedSize;
                }
            }

        CheckEscape();
		} while (!SearchOut.bAborted && (headerPos < fileSize));

	fclose(file);

	UpdateStatus(szArchiveName);
	}
