/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                             *
 *  SEARCH v2.0 (mingw32)          See "copyright.txt"         *
 *  ~~~~~~~~~~~~~~~~~~~~~          for copyright details       *
 *  by Bernhard Tschirren                                      *
 *                                                             *
 *                                                             *
 *  SearchZIP.c:                                               *
 *      Search inside ZIP archives. File format details        *
 *      obtained from Wotsit's File Format Collection,         *
 *      (http://www.wotsit.org/).                              *
 *                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "search.h"


typedef struct
	{
	DWORD signature		ATTRIB_PACKED;
	WORD  version    	ATTRIB_PACKED;
	WORD  flags      	ATTRIB_PACKED;
	WORD  method     	ATTRIB_PACKED;
    WORD  date          ATTRIB_PACKED;
    WORD  time          ATTRIB_PACKED;
	DWORD crc32      	ATTRIB_PACKED;
	DWORD packSize   	ATTRIB_PACKED;
	DWORD origSize   	ATTRIB_PACKED;
	WORD  nameLen    	ATTRIB_PACKED;
	WORD  extraLen   	ATTRIB_PACKED;
	} HEADER;

#define SIGNATURE   	0x04034B50


VOID STDCALL SearchZIP(LPCTSTR szArchiveName)
	{
	FILE    *file;
	HEADER   header;
    DWORD    headerPos;
	CHAR     szFullName[MAX_PATH+1];
	PCHAR    szBaseName;

	DisplayStatus(szArchiveName);

	file = fopen(szArchiveName,"rb");
	if (!file) return;


    headerPos = 0;

	do  {
        /* Locate file header */
        if (fseek(file,headerPos,SEEK_SET) != 0)
            break;

        /* Read file header */
        if (fread(&header,sizeof(HEADER),1,file) != 1)
			break;

		/* Check validity of file */
		if (header.signature != SIGNATURE)
			break;

		/* Read filename */
        if (fread(szFullName,header.nameLen,1,file) != 1)
			break;

		/* Administrivia */
		szFullName[header.nameLen] = '\0';
		FormatPath(szFullName);
        SearchOut.dwNumSearched++;

		/* Strip path */
		szBaseName = StrRScan(szFullName,'\\');
        if (!szBaseName) szBaseName = szFullName; else szBaseName++;

		/* Add to found list (if appropriate) */
		if (FileNameMatch(SearchIn.szSpecs,szBaseName))
			{
			FILETIME fileTime;
			DosDateTimeToFileTime(header.date,header.time,&fileTime);

			AddToFoundList(szFullName,
						   szArchiveName,
						   0,
						   &fileTime,
						   &fileTime,
						   &fileTime,
						   0,
						   header.origSize);
			}

        headerPos += sizeof(HEADER);
        headerPos += header.nameLen;
        headerPos += header.extraLen;
        headerPos += header.packSize;

		CheckEscape();
		} while (!SearchOut.bAborted);

	fclose(file);

	UpdateStatus();
	}
